""" Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
"""
Django settings for sealab project.

Generated by 'django-admin startproject' using Django 1.8.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import glob
import os, subprocess
import sys
from datetime import timedelta
from configparser import RawConfigParser
from os.path import join
from utility.config_parser import all_config

ALL_CONFIG = all_config
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

FIXTURE_DIRS = [
    BASE_DIR + '/sealab/fixtures/'
]

# Load configurations
CONFIG_DIR = join(BASE_DIR, 'sealab')
CONFIG_FILE = 'config.ini'
REDIS_PASSWORD_DEFAULT = 'default_password'

application_config = RawConfigParser()
try:
    application_config.read(join(CONFIG_DIR, CONFIG_FILE))
except ImportError as ie:
    raise (ie)

# TODO consolidate the two strategies below
if not 'test' in sys.argv:
    APP_HOST = application_config.get('application', 'APP_HOST')
    SMTP_HOST = application_config.get('email', 'SMTP_HOST')
    EMAIL_PORT = application_config.getint('email', 'EMAIL_PORT')
    EMAIL_HOST_USER = application_config.get('email', 'EMAIL_HOST_USER')
    EMAIL_HOST_PASSWORD = application_config.get('email', 'EMAIL_HOST_PASSWORD')
    EMAIL_USE_TLS = application_config.getboolean('email', 'EMAIL_USE_TLS')
    EMAIL_TIMEOUT = application_config.getint('email', 'EMAIL_TIMEOUT')
    DEFAULT_FROM_EMAIL = application_config.get('email', 'DEFAULT_FROM_EMAIL')
    DB_HOST = ALL_CONFIG.config.get('postgres').get('pg_host')
    DB_NAME = application_config.get('database', 'DB_NAME')
    DB_USER = ALL_CONFIG.config.get('postgres').get('pg_user')
    DB_PASSWORD = ALL_CONFIG.config.get('postgres').get('pg_password')
    REDIS_PASSWORD = ALL_CONFIG.get('redis', {}).get('redis_password')
    REDIS_HOST = ALL_CONFIG.get('redis', {}).get('redis_host', '127.0.0.1')
    REDIS_PORT = ALL_CONFIG.get('redis', {}).get('redis_port', '6379')
    REDIS_HOST_PRIMARY = ALL_CONFIG.get('redis_primary', {}).get('redis_host_primary', '127.0.0.1')
    REDIS_PASSWORD_PRIMARY = ALL_CONFIG.get('redis_primary', {}).get('redis_password_primary')
    REDIS_PORT_PRIMARY = ALL_CONFIG.get('redis_primary', {}).get('redis_port_primary', '6379')
    REDIS_DB = application_config.get('redis', 'REDIS_DB')
    BEAT_REDIS_DB = application_config.get('redis', 'BEAT_REDIS_DB')
    CELERYD_TASK_TIME_LIMIT = application_config.getint('celeryd', 'CELERYD_TASK_TIME_LIMIT')
    CELERYD_TASK_SOFT_TIME_LIMIT = application_config.getint('celeryd', 'CELERYD_TASK_SOFT_TIME_LIMIT')
    COPY_ENV_FOR_REFERENCE_WORKFLOW = application_config.getboolean('celeryd', 'COPY_ENV_FOR_REFERENCE_WORKFLOW',
                                                                    fallback=True)
    CELERY_TASK_RESULT_EXPIRES = eval(application_config.get('celeryd', 'CELERY_TASK_RESULT_EXPIRES', fallback='7200'))
    DEBUG = application_config.getboolean('application', 'DEBUG')
    ALLOWED_HOSTS = eval(application_config.get('application', 'ALLOWED_HOSTS'))
    SCHEDULES_AUTH_CHECK = application_config.getboolean('application', 'SCHEDULES_AUTH_CHECK', fallback=True)
    LOG_REQUEST_IN_WORKFLOW = application_config.getboolean('application', 'LOG_REQUEST_IN_WORKFLOW', fallback=True)
    LOG_PURGE_CHUNK_SIZE = eval(application_config.get('application', 'LOG_PURGE_CHUNK_SIZE', fallback='1000'))
    THREAD_POOL_WORKER = eval(application_config.get('application', 'THREAD_POOL_WORKER', fallback='32'))
    PARALLEL_BRANCH_THREAD_POOL = eval(application_config.get('application', 'PARALLEL_BRANCH_THREAD_POOL', fallback='4'))
    SYNC_DELAY_LIMIT = eval(application_config.get('application', 'SYNC_DELAY_LIMIT', fallback='60'))
    PARALLEL_PATH = application_config.getboolean('application', 'PARALLEL_PATH', fallback=True)
    MANUAL_INPUT_ESCALATION_SCHEDULE = eval(application_config.get('application', 'MANUAL_INPUT_ESCALATION_SCHEDULE', fallback="{'minute': '0', 'hour': '*/1', 'day_of_week': '*', 'day_of_month': '*', 'month_of_year': '*'}"))
    CYOPS_NULL = application_config.get('application', 'CYOPS_NULL', fallback='CYOPS_NULL')
else:
    APP_HOST = 'localhost'
    SMTP_HOST = 'localhost'
    EMAIL_PORT = 25
    EMAIL_HOST_USER = ''
    EMAIL_HOST_PASSWORD = ''
    EMAIL_USE_TLS = False
    EMAIL_TIMEOUT = 30
    DEFAULT_FROM_EMAIL = 'soc@cybersponse.com'
    DB_HOST = 'localhost'
    DB_NAME = 'sealab'
    DB_USER = 'cyberpgsql'
    DB_PASSWORD = ''
    REDIS_HOST = 'localhost'
    REDIS_PORT = '6379'
    CELERYD_TASK_TIME_LIMIT = 600
    CELERYD_TASK_SOFT_TIME_LIMIT = 600
    COPY_ENV_FOR_REFERENCE_WORKFLOW = False
    CELERY_TASK_RESULT_EXPIRES = 7200
    DEBUG = True
    ALLOWED_HOSTS = []
    SCHEDULES_AUTH_CHECK = True
    LOG_REQUEST_IN_WORKFLOW = False
    PARALLEL_PATH = True
    MANUAL_INPUT_ESCALATION_SCHEDULE = {'month_of_year': '*', 'day_of_month': '*', 'hour': '*/1', 'minute': '0', 'day_of_week': '*'}
    PARALLEL_BRANCH_THREAD_POOL = 4
    CYOPS_NULL = 'CYOPS_NULL'

# TODO: this is really stupid
# this part should go in wsgi.py and/or manage.py, but celery needs it too, and
# if it lives here then env args will always be set reguardless of how the
# application was started and doesn't have to be repeated in three different
# places.
ENV_DIR = os.path.join(BASE_DIR, '.envdir')
ENV_VARS = glob.glob(os.path.join(ENV_DIR, '*'))
for env_var in ENV_VARS:
    with open(env_var, 'r') as env_var_file:
        os.environ.setdefault(env_var.split(os.sep)[-1],
                              env_var_file.read().strip())

APPLIANCE_PUBLIC_KEY = os.getenv('APPLIANCE_PUBLIC_KEY', '')
APPLIANCE_PRIVATE_KEY = os.getenv('APPLIANCE_PRIVATE_KEY', '')
SEALAB_PUBLIC_KEY = os.getenv('SEALAB_PUBLIC_KEY', '')
SEALAB_PRIVATE_KEY = os.getenv('SEALAB_PRIVATE_KEY', '')

# RabbitMQ Publisher
RABBITMQ_CONFIG_FILE = "/opt/cyops/configs/rabbitmq/rabbitmq_users.conf"
CONNECTION_RETRY_COUNT = 3
CONNECTION_RETRY_DELAY = 5
PUBLISH_MASSAGE_RETRY_COUNT = 5
ACTION_LIMIT_ENABLED = False

# ToDo  can be moved in config file ?
QUEUE = {
    'queue_sealab_action_remoterequest': {
        'exchangeName': 'texchange.cyops.sealab',
        'routingKey': 'route.sealab.action.remoterequest'
    },
    'queue_sealab_action_remoteresponse': {
        'exchangeName': 'texchange.cyops.sealab',
        'routingKey': 'route.sealab.action.remoteresponse'
    }
}


# Auth settings
HMAC_TIME_THRESHOLD = 30
HMAC_USER = 'hmac_user'
BASIC_USER = 'basic_user'
ANONYMOUS_USER = 'anon_user'

# Celery settings
# BROKER_URL = 'mongodb://localhost:27017/venom'

if 'test' in sys.argv:
    REDIS_DB = '8'  # noqa
    BEAT_REDIS_DB = '9'  # noqa

if 'test' not in sys.argv:
    import redis
    if REDIS_PASSWORD_PRIMARY and REDIS_PASSWORD_PRIMARY != REDIS_PASSWORD_DEFAULT:
        output = subprocess.check_output(
            ['/opt/cyops-auth/.env/bin/python', '/opt/cyops/configs/scripts/manage_passwords.py', '--decrypt',
             REDIS_PASSWORD_PRIMARY, 'jQp3(7@jod#j38d1']
            ).strip().decode()
        if not output.startswith('Password:'):
            raise Exception('DB Password decrypt failed with error %s' % output)
        else:
            REDIS_PASSWORD_PRIMARY = output[len('Password:'):]
        REDIS_CONN = redis.StrictRedis(REDIS_HOST_PRIMARY, REDIS_PORT_PRIMARY, REDIS_DB, password=REDIS_PASSWORD_PRIMARY)
        BEAT_REDIS_CONN = redis.StrictRedis(REDIS_HOST_PRIMARY, REDIS_PORT_PRIMARY, BEAT_REDIS_DB, password=REDIS_PASSWORD_PRIMARY)
        CELERY_RESULT_BACKEND = 'redis://:' + REDIS_PASSWORD_PRIMARY + '@' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + \
                                REDIS_DB
        CELERYBEAT_REDIS_URL = ('redis://:' + REDIS_PASSWORD_PRIMARY + '@' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' +
                                BEAT_REDIS_DB)
        BROKER_URL = 'redis://:' + REDIS_PASSWORD_PRIMARY + '@' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + REDIS_DB
    else:
        REDIS_CONN = redis.StrictRedis(REDIS_HOST_PRIMARY, REDIS_PORT_PRIMARY, REDIS_DB)
        BEAT_REDIS_CONN = redis.StrictRedis(REDIS_HOST_PRIMARY, REDIS_PORT_PRIMARY, BEAT_REDIS_DB)
        CELERY_RESULT_BACKEND = 'redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + \
                                REDIS_DB
        CELERYBEAT_REDIS_URL = ('redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' +
                                BEAT_REDIS_DB)
        BROKER_URL = 'redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + REDIS_DB

else:
    import fakeredis

    REDIS_CONN = fakeredis.FakeStrictRedis()
    BEAT_REDIS_CONN = fakeredis.FakeStrictRedis()
    CELERY_ALWAYS_EAGER = True
    CELERY_EAGER_PROPAGATES_EXCEPTIONS = True
    CELERY_RESULT_BACKEND = 'redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + \
                                REDIS_DB
    CELERYBEAT_REDIS_URL = ('redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' +
                                BEAT_REDIS_DB)
    BROKER_URL = 'redis://' + REDIS_HOST_PRIMARY + ':' + REDIS_PORT_PRIMARY + '/' + REDIS_DB

#: Only add pickle to this list if your broker is secured
#: from unwanted access (see userguide/security.html)
CELERY_ACCEPT_CONTENT = ['json']
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'

CELERY_IMPORTS = (
    'celery.task'
)


CELERYBEAT_SYNC_EVERY = 5
# from celerybeat redis
CELERY_REDIS_SCHEDULER_URL = CELERYBEAT_REDIS_URL
CELERY_REDIS_SCHEDULER_KEY_PREFIX = 'tasks:meta:'

TEST_RUNNER = 'rainbowtests.test.runner.RainbowDiscoverRunner'

# rest framework settings
REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated'
    ],
    'DEFAULT_AUTHENTICATION_CLASSES': [
        'rest_framework.authentication.SessionAuthentication',
        'auth_ogre.schemes.HmacAuthenticationScheme'
    ],
    'DEFAULT_RENDERER_CLASSES': [
        # 'rest_framework.renderers.BrowsableAPIRenderer',
        'rest_framework.renderers.JSONRenderer',
    ],
    # pagination
    'DEFAULT_PAGINATION_CLASS':
        'lerna.pagination.HydraLimitOffsetPagination',
    'PAGE_SIZE': 10,
    'DEFAULT_METADATA_CLASS': 'rest_framework.metadata.SimpleMetadata',
}
ANONYMOUS_USER_ID = -1

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'yco#qath+mim6sfb&$zcfye25ph2@3725ak$ktp$$7q8^@4)@-'

# Application definition

INSTALLED_APPS = (
    # django
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # 3ed party
    'rest_framework',
    'django_extensions',
    'memoize',
    'django_filters',
    'crispy_forms',

    # cs
    'workflow',
    'lerna',
    'rulegin',
    'dynamic_variable',
    'ansible_filters',
    'django_celery_beat',
    'expression_builder',
)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = 'sealab.urls'

TEMPLATES = [
    {
        'NAME': 'django',
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
    {
        'NAME': 'jinja2',
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [],
        'APP_DIRS': False,
        'OPTIONS': {
            'environment': 'sealab.jinja.environment'
        }
    },
]

CACHE_OPTIONS = {
            'DB': 11,
            'PARSER_CLASS': 'redis.connection.HiredisParser',
            'CONNECTION_POOL_CLASS': 'redis.BlockingConnectionPool',
            'CONNECTION_POOL_CLASS_KWARGS': {
                'max_connections': 50,
                'timeout': 20,
            },
            'MAX_CONNECTIONS': 1000,
            'PICKLE_VERSION': -1,
        }
if REDIS_PASSWORD and REDIS_PASSWORD != REDIS_PASSWORD_DEFAULT:
    output = subprocess.check_output(
        ['/opt/cyops-auth/.env/bin/python', '/opt/cyops/configs/scripts/manage_passwords.py', '--decrypt',
         REDIS_PASSWORD, 'jQp3(7@jod#j38d1']
    ).strip().decode()
    if not output.startswith('Password:'):
        raise Exception('DB Password decrypt failed with error %s' % output)
    else:
        REDIS_PASSWORD = output[len('Password:'):]
    CACHE_OPTIONS['PASSWORD'] = REDIS_PASSWORD

CACHES = {
    'default': {
        'BACKEND': 'redis_cache.RedisCache',
        'LOCATION': [
            REDIS_HOST + ':' + REDIS_PORT
        ],
        'OPTIONS': CACHE_OPTIONS,
    },
}

WSGI_APPLICATION = 'sealab.wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

# Decrypt DB password
if DB_PASSWORD:
    output = subprocess.check_output(['/opt/cyops-auth/.env/bin/python', '/opt/cyops/configs/scripts/manage_passwords.py', '--decrypt',
                                      DB_PASSWORD, 'jQp3(7@jod#j38d1']
                                     ).strip().decode()
    if not output.startswith('Password:'):
        raise Exception('DB Password decrypt failed with error %s' % output)
    else:
        DB_PASSWORD = output[len('Password:'):]

if 'test' in sys.argv:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'USER': 'postgres',
            'PASSWORD': DB_PASSWORD,
            'NAME': 'sealab_testz',
            'HOST': DB_HOST
        }
    }
else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'USER': DB_USER,
            'PASSWORD': DB_PASSWORD,
            'NAME': DB_NAME,
            'HOST': DB_HOST
        }
    }

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/
WEB_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'web/')

STATIC_URL = '/wf/static/'
STATIC_ROOT = os.path.join(WEB_ROOT, 'static/')

if DEBUG:
    DOCS_ROOT = os.path.join(BASE_DIR, '../docs/build/html/')
    STATICFILES_DIRS = (
        ('docs', DOCS_ROOT),
    )

MEDIA_URL = '/wf/media/'
MEDIA_ROOT = os.path.join(WEB_ROOT, 'media/')

TMP_FILE_ROOT = '/tmp/'

if 'uwsgi' in sys.argv:
    LOGFILE_LOC = '/var/log/cyops/cyops-workflow/sealab.log'
else:
    LOGFILE_LOC = os.path.join(os.path.dirname(BASE_DIR),
                               'sealab.log')

try:
    from sealab.config import WORKFLOW_LOG_LEVEL
except ImportError as ie:
    WORKFLOW_LOG_LEVEL = 'WARNING'

# Only configure logging when running Django, not celery
if 'worker' not in sys.argv and 'beat' not in sys.argv:
    LOGGING = {
        'version': 1,
        'disable_existing_loggers': False,
        'handlers': {
            # Log to a text file that can be rotated by logrotate
            'logfile': {
                'class': 'logging.handlers.WatchedFileHandler',
                'filename': LOGFILE_LOC,
                'formatter': 'verbose'
            }
        },
        'formatters': {
            'verbose': {
                'format': "[%(asctime)s] %(levelname)s " +
                          "[%(name)s:%(lineno)s] %(message)s",
                'datefmt': "%d/%b/%Y %H:%M:%S"
            },
            'simple': {
                'format': '%(levelname)s %(message)s'
            },
        },
        'loggers': {
            'django': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'workflow': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'dynamic_variable': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'expression_builder': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'rulegin': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            },
            'django.db.backends': {
                'handlers': ['logfile'],
                'level': WORKFLOW_LOG_LEVEL,
                'propagate': False
            }
        },
    }

# TODO find a better place to put these
CRUD_HUB_URL = 'https://' + APP_HOST
CRUD_HUB_API_PREFIX = '/api/3'
QRADAR_HOST = 'https://example.com'
QRADAR_TOKEN = 'not_real'

EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

if DEBUG:
    # Forward to mailcatcher server
    EMAIL_HOST = '127.0.0.1'
    EMAIL_PORT = 1025

else:
    EMAIL_HOST = SMTP_HOST

TIME_FORMAT = '%Y-%m-%d %H:%M:%S'

INTEGRATIONS_URL = 'https://' + APP_HOST + ':9595/integration/execute/?format=json'
