""" Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import sys
import eventlet, requests, time
import logging, os, pika, json

from django.apps import AppConfig
from django.conf import settings
from expression_builder.exp import prepare_exp_list
from auth_ogre.requests_auth import get_requests_auth
from sealab.utils import maybe_json_or_raise
from workflow.errors.error_constants import *



logger = logging.getLogger(__name__)


class WorkflowConfig(AppConfig):
    name = 'workflow'
    verbose_name = "Workflow"

    def ready(self):
        try:

            try:
                if 'uwsgi' in sys.argv:
                    settings.EXPRESSIONS = prepare_exp_list()
            except Exception as e:
                logger.warn('Unable to populate expressions due to error: %s', str(e))

            if 'eventlet' in sys.argv:
                eventlet.monkey_patch()
            logger.info('Updating proxy settings for the environment')
            if 'uwsgi' in sys.argv or 'worker' in sys.argv or 'beat' in sys.argv:
                configure_proxy()
                settings.ACTION_LIMIT_ENABLED = is_action_count_restriction_enabled(settings.CRUD_HUB_URL)
            import workflow.signals
        except Exception as e:
            logger.exception(str(e))


def configure_proxy(system_settings=None):
    try:
        config = {}
        if not system_settings:
            collection = settings.CRUD_HUB_URL + '/api/3/proxy'
            method = 'GET'
            auth_info = {'auth_method': 'CS HMAC'}
            try:
                auth = get_requests_auth(auth_info, collection, method,
                                           settings.APPLIANCE_PUBLIC_KEY,
                                           settings.APPLIANCE_PRIVATE_KEY)

                response = requests.request(method, collection, auth=auth, verify=False)
                system_settings = maybe_json_or_raise(response)
            except Exception as e:
                logger.error('Error while fetching proxy details %s', str(e))
                system_settings = None

        if system_settings:
            for k, v in system_settings.get('proxies', {}).items():
                config.update({k: v.get('value', '')})

            NO_PROXY = ','.join(system_settings.get('noProxy', []))
            if NO_PROXY:
                logger.info('Setting env variable %s', 'NO_PROXY')
                os.environ['NO_PROXY'] = NO_PROXY

            for el in system_settings.get('OTHER', []):
                key = list(el.keys())[0]
                logger.info('Setting env variable %s', key)
                os.environ[key] = el.get(key)

            for k, v in config.items():
                if v and system_settings.get('proxies', {}).get(k, {}).get('is_enabled'):
                    logger.info('Setting env variable %s', k)
                    os.environ[k] = v
                else:
                    logger.info('Clearing env variable %s', k)
                    os.environ.pop(k, None)
    except Exception as e:
        logger.exception(cs_wf_31.format(str(e)))


def is_action_count_restriction_enabled(app_url, retry_count=5, retry_interval=5):
    app_url = app_url+'/auth/license/?param=license_details'
    try:
        license_details = requests.get(app_url, verify=False).json()
        daily_action_limit = license_details.get('details').get('entitlements').get('daily_action_limit')
        if daily_action_limit is not None and daily_action_limit > 0:
            return True
    except Exception:
        logger.error('Could not retrieve license details from DAS')
        if retry_count > 1:
            time.sleep(retry_interval)
            logger.error('Retrying...')
            is_action_count_restriction_enabled(app_url, retry_count-1)
    return False
